#! /bin/sh
#-----------------------------------------------------------------------------
# /tmp/install.sh - install postgrsql package
#
# Copyright (c) 2004 Frank Meyer <frank@eisfair(dot)org>
#
# Creation:    30.07.2004 fm
# Last update: $Id: install.sh 27965 2011-04-06 19:48:35Z dv $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#-----------------------------------------------------------------------------

# include eislib 
. /var/install/include/eislib

# setup system specific values	
if [ -z "${EISFAIR_SYSTEM}" ]
then
    # get eisfair system-type
    . /var/install/include/check-eisfair-version
fi 
 	
case "${EISFAIR_SYSTEM}" in
    eisfair-1)
        runuser="postgres"
        runuid="26"
        rungroup="postgres"
        rungid="26"
        ;;
    *)
        runuser="postgres"
        runuid="126"
        rungroup="postgres"
        rungid="126"
        ;;
esac

data_dir='/var/lib/pgsql'
inst_dir='/usr/local/pgsql'


#-----------------------------------------------------------------------------
# show wait message to entertain the user
#-----------------------------------------------------------------------------

function show_wait
{
    mecho -n "wait [ "
    colpos=1
    while [ -n "`ps --no-headers $!`" ]
    do
        mecho -info -n "."
        sleep 1
        colpos=$[$colpos + 1]  
        if [ "$colpos" -ge 55 ]
        then
            mecho " ]"
            mecho -n "wait [ "
            colpos=1
        fi
    done 
    mecho " ] done"
}


#-----------------------------------------------------------------------------
# ensure that postgresql is running
#-----------------------------------------------------------------------------

function bring_server_up()
{
    PG_CTL="${inst_dir}/bin/pg_ctl"

    /bin/su ${runuser} -c "$PG_CTL -w -D ${data_dir} status" 2>1 1>/dev/null
    if [ "$?" != "0" ]
    then
        mecho -info "PostgreSQL server is not running. Trying to start..."

        /bin/su postgres -c "$PG_CTL -s -w -D /var/lib/pgsql start" \
           > /dev/null 2> /dev/null

        if [ "$?" != "0" ]
        then
            mecho -error "Unable to start PostgreSQL server!"
        fi
    fi
}


#-----------------------------------------------------------------------------
# ensure that postgresql is shut down
#-----------------------------------------------------------------------------

function bring_server_down()
{
    PG_CTL="${inst_dir}/bin/pg_ctl"

    /bin/su ${runuser} -c "$PG_CTL -w -D ${data_dir} status" 2>1 1>/dev/null  
    if [ "$?" == "0" ]
    then
        mecho -info "PostgreSQL server is running. Shutting down..."

        /bin/su postgres -c "$PG_CTL -s -w -D /var/lib/pgsql stop -m fast" \
           > /dev/null 2> /dev/null 

        if [ "$?" != "0" ]
        then
            mecho -error "Unable to stop PostgreSQL server!"
        fi
    fi
}

#-----------------------------------------------------------------------------
# start/stop script
#-----------------------------------------------------------------------------
stopplevels="/etc/rc0.d /etc/rc1.d /etc/rc6.d"
startlevels="/etc/rc2.d"

case "${EISFAIR_SYSTEM}" in
    eisfair-1)
        cd /etc/rc2.d
        rm -f K??postgresql
        rm -f S??postgresql
        ln -sf ../init.d/postgresql K55postgresql
        cd - >/dev/null 2>&1
        ;;
    *)
        for level in $stopplevels
        do
            if [ -d "$level" ]
            then
                cd "$level"
                rm -f K??postgresql
                ln -sf ../init.d/postgresql K55postgresql
                cd - >/dev/null 2>&1
            fi
        done

        for level in $startlevels
        do
            if [ -d "$level" ]
            then
                cd "$level"
                rm -f K??postgresql
                cd - >/dev/null 2>&1
            fi
        done
        ;;
esac

#-----------------------------------------------------------------------------
# create symlinks
#-----------------------------------------------------------------------------
cd ${inst_dir}/bin
ln -sf postgres postmaster
cd - >/dev/null 2>&1

cd ${inst_dir}/lib
rm -f libecpg.so
rm -f libecpg.so.6
rm -f libecpg_compat.so
rm -f libecpg_compat.so.3
rm -f libpgtypes.so
rm -f libpgtypes.so.3
ln -sf libecpg.so.6.1 libecpg.so
ln -sf libecpg.so.6.1 libecpg.so.6
ln -sf libecpg_compat.so.3.1 libecpg_compat.so
ln -sf libecpg_compat.so.3.1 libecpg_compat.so.3
ln -sf libpgtypes.so.3.1 libpgtypes.so
ln -sf libpgtypes.so.3.1 libpgtypes.so.3
cd -  >/dev/null 2>&1

#-----------------------------------------------------------------------------
# postgres user and group
#-----------------------------------------------------------------------------
if [ -z "`grep ${rungroup}: /etc/group`" ] 
then  
    /var/install/bin/add-group $rungroup $rungid >/dev/null
fi 
if [ -z "`grep ${runuser}: /etc/passwd`" ] 
then  
    echo ""  
    mecho -info "Adding user $runuser ..." 
    /var/install/bin/add-user $runuser "*" $runuid $rungid "PostgreSQL daemon" $inst_dir /bin/sh >/dev/null
fi

#-----------------------------------------------------------------------------
# uninstall links that have been left by previous uninstallers
#-----------------------------------------------------------------------------
rm -f /lib/libpq.so
rm -f /lib/libpq.so.4
rm -f /lib/libpq.so.4.0

#-----------------------------------------------------------------------------
# remove old log files that are now kept in /var/lib/pgsql/log
#-----------------------------------------------------------------------------
rm -f /var/log/postmaster*

#-----------------------------------------------------------------------------
# initialize database cluster in /var/lib/pgsql
#-----------------------------------------------------------------------------
if [ ! -d ${data_dir} ]
then
    # start initdb
    mecho -info "Setting up postgres files:"
    mkdir -p ${data_dir}
    chown ${runuser}.${rungroup} ${data_dir}
    su ${runuser} -c "${inst_dir}/bin/initdb --locale=C --encoding=LATIN9 -D ${data_dir}" >/tmp/$$ 2>&1 &

    # entertain the user
    show_wait

    # evaluate exit code
    wait $!
    if [ "$?" != "0" ]
    then
	cat /tmp/$$
    fi
    rm -f /tmp/$$
    
    # fix file permissions
    chown root.root ${inst_dir}
    chmod 0755 ${inst_dir}
fi

#-----------------------------------------------------------------------------
# menu
#-----------------------------------------------------------------------------
/var/install/bin/add-menu setup.services.menu             \
                          setup.services.postgresql.menu  \
                          "PostgreSQL Database Server"

#-----------------------------------------------------------------------------
# restore data
#-----------------------------------------------------------------------------

if [ -f /tmp/postgresql.dumpdir ]
then
    read dumpdir < /tmp/postgresql.dumpdir
    archive="$dumpdir/postgresql.dump.gz"

    if [ -f "$archive" ]
    then
        bring_server_up

        # reload data
        mecho -info "Reloading old data. This may take quite a long time..."
        mecho -warn "DON'T INTERRUPT THIS PROCESS UNDER ANY CIRCUMSTANCES!"
        mecho -warn "EVEN IF THE SCRIPT SEEMS TO BE FROZEN!"
        zcat $archive | ${inst_dir}/bin/psql -U postgres -d postgres > /dev/null 2> /dev/null &

        # entertain the user
        show_wait

        # evaluate exit code
        wait $!
        if [ "$?" != "0" ]
        then
            mecho -warn "Reloading completed with errors!"
            echo "Please check the data of your databases. If there is something missing"
            echo "you find your data in \"$archive\" (compressed text file)."
            echo "If everything is there, it is save to remove the dump file:"
            echo "  rm -f $archive"
            anykey
        else
            rm -f $archive
        fi

        # shutdown server
        bring_server_down
    fi
    rm -f /tmp/postgresql.dumpdir
fi

#-----------------------------------------------------------------------------
# do configuration update
#-----------------------------------------------------------------------------

/var/install/config.d/postgresql-update.sh
/var/install/config.d/postgresql.sh
 
#-----------------------------------------------------------------------------
# start server if configured so
#-----------------------------------------------------------------------------
if [ -f /etc/rc2.d/S45postgresql ]
then
    /etc/init.d/postgresql start
fi

exit 0
