#! /bin/sh
#-----------------------------------------------------------------------------
# /tmp/preinstall.sh - install postgrsql package
#
# Copyright (c) 2004 Frank Meyer <frank@eisfair(dot)org>
#
# Creation:    30.07.2004 fm
# Last update: $Id: preinstall.sh 24512 2010-06-12 21:17:43Z dv $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#-----------------------------------------------------------------------------

. /var/install/include/eislib

dumpdir=""

if tar --help | grep -q 'no-overwrite-dir'
then
    taroption='--no-overwrite-dir'
else
    taroption=''
fi


#-----------------------------------------------------------------------------
# show wait message to entertain the user
#-----------------------------------------------------------------------------

function show_wait
{
    mecho -n "wait [ "
    colpos=1
    while [ -n "`ps --no-headers $!`" ]
    do
        mecho -info -n "."
        sleep 1
        colpos=$[$colpos + 1]
        if [ "$colpos" -ge 55 ]
        then
            mecho " ]"
            mecho -n "wait [ "
            colpos=1
        fi
    done  
    mecho " ] done"
}


#-----------------------------------------------------------------------------
# Print error message and exit
#-----------------------------------------------------------------------------

failed_update()
{
    mecho -error "Update failed! Please try the following procedure:"
    echo "1. Make a complete backup of the databases you want to keep."
    echo "   Use pg_dump or other utilies to perform this task. Please"
    echo "   do not copy the database directory by hand."
    echo "2. Uninstall the PostgreSQL database package currently installed."
    echo "3. Install the new PostgreSQL package."
    echo "4. Create the databases and users and restore the contents of"
    echo "   the databases using pg_restore or other utilties."
    echo ""
    mecho -warn "Installation aborted!"
    anykey
    exit 1
}


#-----------------------------------------------------------------------------
# extract to most current pg_dumpall program from archive
#-----------------------------------------------------------------------------

extract_dump_program()
{
    archive="/tmp/postgresql.tar.gz"
    archivepath="usr/local/pgsql/bin"

    if [ ! -f "$archive" ]
    then
        mecho -error "Archive not found!"
        failed_update
    fi

    dumpprog="pg_dumpall"

    # extract pg_dumpall
    mecho -info "Extracting \"$dumpprog\" from archive:"
    zcat $archive | tar $taroption -x -f - "$archivepath/$dumpprog" -O > /tmp/$dumpprog 2>/dev/null &

    # entertain the user
    show_wait

    # evaluate exit code
    wait $!
    if [ "$?" != "0" ]
    then
        mecho -error "Error extracting \"$dumpprog\"! Not found? No space left on device?"
        rm -f /tmp/$dumpprog
        failed_update
    else
        chmod a+x /tmp/$dumpprog
    fi

    dumpprog="pg_dump"

    # extract pg_dump
    mecho -info "Extracting \"$dumpprog\" from archive:"
    zcat $archive | tar $taroption -x -f - "$archivepath/$dumpprog" -O > /tmp/$dumpprog 2>/dev/null &

    # entertain the user
    show_wait

    # evaluate exit code
    wait $!
    if [ "$?" != "0" ]
    then
        mecho -error "Error extracting \"$dumpprog\"! Not found? No space left on device?"
        rm -f /tmp/$dumpprog
        failed_update
    else
        chmod a+x /tmp/$dumpprog
    fi
}


#-----------------------------------------------------------------------------
# ensure that postgresql is running
#-----------------------------------------------------------------------------

bring_server_up()
{
    PG_CTL='/usr/local/pgsql/bin/pg_ctl'

    /bin/su postgres -c "$PG_CTL -w -D /var/lib/pgsql status" 2>1 1>/dev/null
    if [ "$?" != "0" ]
    then
        mecho -info "PostgreSQL server is not running. Trying to start..."

        /etc/init.d/postgresql start
        if [ "$?" != "0" ]
        then
            mecho -error "Unable to start PostgreSQL server!"
            failed_update
        fi
    fi
}


#-----------------------------------------------------------------------------
# ensure that postgresql is down
#-----------------------------------------------------------------------------

bring_server_down()
{
    /etc/init.d/postgresql stop
    if [ "$?" != "0" ]
    then
        mecho -error "Unable to shut down PostgreSQL server!"
        failed_update
    fi
}


#-----------------------------------------------------------------------------
# perform a dump of the previous database to /tmp/postgresql.dump.gz
#-----------------------------------------------------------------------------

dump_all_data()
{
    # dump all data
    mecho -info "Dumping to $dumpdir/postgresql.dump.gz:"
    /tmp/pg_dumpall -o -U postgres | gzip > "$dumpdir/postgresql.dump.gz" &

    # entertain the user
    show_wait

    # evaluate exit code
    wait $!
    if [ "$?" != "0" -a -f "/$dumpdir/postgresql.dump.gz" ]
    then
        mecho -error "Error dumping data! No space left on device?"
        rm -f $dumpdir/postgresql.dump.gz
        failed_update
    else
        rm -f /tmp/pg_dumpall
        rm -f /tmp/pg_dump
    fi
}


#-----------------------------------------------------------------------------
# perform update
#-----------------------------------------------------------------------------

dump_data()
{
    rm -f /tmp/postgresql.dump.gz
    rm -f /tmp/postgresql.dumpdir

    dumpdir=""
    while [ "$dumpdir" == "" ]
    do
       echo -n "Specify directory to dump data to: [/tmp] " 
       read dumpdir

       if [ "$dumpdir" == "" ]
       then
           dumpdir="/tmp"
       else
           if [! -d "$dumpdir" ]
           then
               mecho -warn "\"$dumpdir\" is not a valid directory!"
               dumpdir=""
           fi
       fi
    done

    echo $dumpdir > /tmp/postgresql.dumpdir

    extract_dump_program
    bring_server_up
    dump_all_data $dumpdir
    bring_server_down

    rm -rf /var/lib/pgsql
    if [ "$?" != "0" ]
    then
        mecho -error "Unable to remove old database data. This is a critical error!"
        mecho -error "DO NOT RETRY THE UPDATE PROCEDURE!"
        echo "Please perform the following steps instead:"
        echo "1. Uninstall the PostgreSQL package currently installed."
        echo "2. Remove the database directory manually with \"rm -rf /var/lib/pgsql\"."
        echo "3. Install the new package"
        echo "4. Execute the command:"
        echo "   zcat /tmp/postgresql.dump.gz | \\"
        echo "    /usr/local/pgsql/bin/psql -U postgres -d postgres"
        echo "Good luck!"
        echo ""
        mecho -warn "Installation aborted!"
        anykey
        exit 1
    fi
}


#-----------------------------------------------------------------------------
# check version of database directory (in case of updates)
#-----------------------------------------------------------------------------

if [ -f "/var/lib/pgsql/PG_VERSION" ]
then
    test=`cat /var/lib/pgsql/PG_VERSION`
    if [ "$test" == "8.0" -o "$test" == "8.1" -o "$test" == "8.2" -o "$test" == "8.3" ]
    then
        mecho -info "Installation of old version found!"
        echo "The data of your previous installation has to be dumped"
        echo "and reloaded during the update process."
        echo "This is done automatically, but it is recommended to"
        echo "perform a complete backup of you databases first!"
        /var/install/bin/ask "Continue" "y"
        if [ "$?" == 0 ]
        then
            dump_data
        else
            mecho -warn "Installation aborted!"
            anykey
            exit 1
        fi
    else 
        if [ "$test" != "8.4" ]
        then
            mecho -error "Conflicting database versions!"
            echo "The version of PostgreSQL currently installed on your system is"
            echo "not compatible with the database layout of the new version."
            echo "Therefore an automatic update is not possible!"
            echo ""
            echo "To resolve this conflict please perform the following steps:"
            echo "1. Make a complete backup of the databases you want to keep."
            echo "   Use pg_dump or other utilies to perform this task. Please"
            echo "   do not copy the database directory by hand."
            echo "2. Uninstall the PostgreSQL database package currently installed."
            echo "3. Install the new PostgreSQL package."
            echo "4. Create the databases and users and restore the contents of"
            echo "   the databases using pg_restore or other utilties."
            echo ""
            mecho -warn "Installation aborted!"
            anykey
            exit 1
        fi
    fi
fi

#-----------------------------------------------------------------------------
# check for uninstall script of previous versions
# if found, the uninstall routine has to be tweaked so that it does not
# remove the links to the library libpq, since this is a seperate package now
#-----------------------------------------------------------------------------
if [ -f /var/install/deinstall/postgresql ]
then
    cat /var/install/deinstall/postgresql | grep -v "rm -f  /usr/lib/libpq.so" > /tmp/postgresql$$.tmp
    if [ "$?" == "0" ]
    then
        mv /tmp/postgresql$$.tmp /var/install/deinstall/postgresql
    fi
fi

#-----------------------------------------------------------------------------
# start/stop script
#-----------------------------------------------------------------------------

if [ -f /etc/init.d/postgresql ]
then
    sh /etc/init.d/postgresql stop
fi


#-----------------------------------------------------------------------------
# uninstall previous version
#-----------------------------------------------------------------------------

if [ -f /var/install/deinstall/postgresql ]
then
    sh /var/install/deinstall/postgresql update
fi

exit 0
